@@ -1,3 +1,6 @@
+"""
+File containing functions for training a surrogate model in pytorch taking into account the error in nano amperes.
+"""
 import os
 import torch
 import matplotlib.pyplot as plt
@@ -28,12 +31,12 @@ def init_seed(configs: dict) -> None:
     Parameters
     ----------
     configs : dict
-         Training configurations with the following keys:
+        Training configurations with the following keys:
 
-        - seed:  int [Optional]
-            The desired seed for the random number generator. If the dictionary does not contain 
-            this key, a deterministic random seed will be applied, and added to the key 'seed' in 
-            the dictionary.
+        1. seed:  int [Optional]
+        The desired seed for the random number generator. If the dictionary does not contain 
+        this key, a deterministic random seed will be applied, and added to the key 'seed' in 
+        the dictionary.
     """
     if "seed" in configs:
         seed = configs["seed"]
@@ -66,51 +69,63 @@ def generate_surrogate_model(
     ----------
     configs : dict
         Training configurations for training a model with following keys:
-        * results_base_dir: str
-            Directory where the trained model and corresponding performance plots will be stored.
-        * seed: int
-            Sets the seed for generating random numbers to a non-deterministic random number.
-        * hyperparameters:
-            epochs: int
-            learning_rate: float
-        * model_structure: dict
-            The definition of the internal structure of the surrogate model, which is typically five
-            fully-connected layers of 90 nodes each.
-            - hidden_sizes : list
-                A list containing the number of nodes of each layer of the surrogate model.
-                E.g., [90,90,90,90,90]
-            - D_in: int
-                Number of input features of the surrogate model structure. It should correspond to
-                the activation electrode number.
-            - D_out: int
-                Number of output features of the surrogate model structure. It should correspond to
-                the readout electrode number.
-        * data:
-            dataset_paths: list[str]
-                A list of paths to the Training, Validation and Test datasets, stored as
-                postprocessed_data.npz
-            steps : int
-                It allows to skip parts of the data when loading it into memory. The number indicates
-                how many items will be skipped in between. By default, step number is one (no values
-                are skipped). E.g., if steps = 2, and the inputs are [0, 1, 2, 3, 4, 5, 6]. The only
-                inputs taken into account would be: [0, 2, 4, 6].
-            batch_size: int
-                How many samples will contain each forward pass.
-            worker_no: int
-                How many subprocesses to use for data loading. 0 means that the data will be loaded in
-                the main process. (default: 0)
-            pin_memory: boolean
-                If True, the data loader will copy Tensors into CUDA pinned memory before returning
-                them. If your data elements are a custom type, or your collate_fn returns a batch that
-                is a custom type.
-    custom_model : custom model of type torch.nn.Module
-        Model to be trained.
-    criterion : <method>
-        Loss function that will be used to train the model.
-    custom_optimizer : torch.optim.Optimizer
-        Optimization method used to train the model which decreases model's loss.
-    save_dir : string [Optional]
-        Name of the path where the trained model is to be saved.
+        
+        1. results_base_dir: str
+        Directory where the trained model and corresponding performance plots will be stored.
+
+        2. seed: int
+        Sets the seed for generating random numbers to a non-deterministic random number.
+
+        3. hyperparameters:
+        epochs: int
+        learning_rate: float
+        
+        4. model_structure: dict
+        The definition of the internal structure of the surrogate model, which is typically five
+        fully-connected layers of 90 nodes each.
+
+        4.1 hidden_sizes : list
+        A list containing the number of nodes of each layer of the surrogate model.
+        E.g., [90,90,90,90,90]
+
+        4.2 D_in: int
+        Number of input features of the surrogate model structure. It should correspond to
+        the activation electrode number.
+
+        4.3 D_out: int
+        Number of output features of the surrogate model structure. It should correspond to
+        the readout electrode number.
+
+        5. data:
+        5.1 dataset_paths: list[str]
+        A list of paths to the Training, Validation and Test datasets, stored as
+        postprocessed_data.npz
+
+        5.2 steps : int
+        It allows to skip parts of the data when loading it into memory. The number indicates
+        how many items will be skipped in between. By default, step number is one (no values
+        are skipped). E.g., if steps = 2, and the inputs are [0, 1, 2, 3, 4, 5, 6]. The only
+        inputs taken into account would be: [0, 2, 4, 6].
+
+        5.3 batch_size: int
+        How many samples will contain each forward pass.
+
+        5.4 worker_no: int
+        How many subprocesses to use for data loading. 0 means that the data will be loaded in
+        the main process. (default: 0)
+
+        5.5 pin_memory: boolean
+        If True, the data loader will copy Tensors into CUDA pinned memory before returning
+        them. If your data elements are a custom type, or your collate_fn returns a batch that
+        is a custom type.
+        custom_model : custom model of type torch.nn.Module
+            Model to be trained.
+        criterion : <method>
+            Loss function that will be used to train the model.
+        custom_optimizer : torch.optim.Optimizer
+            Optimization method used to train the model which decreases model's loss.
+        save_dir : string [Optional]
+            Name of the path where the trained model is to be saved.
     
     Return
     ------
@@ -210,43 +225,55 @@ def train_loop(
         Model to be trained.
     info_dict : dict
         The dictionary used for initialising the surrogate model. It has the following keys:
-        * model_structure: dict
-            The definition of the internal structure of the surrogate model, which is typically five
-            fully-connected layers of 90 nodes each.
-            - hidden_sizes : list
-                A list containing the number of nodes of each layer of the surrogate model.
-                E.g., [90,90,90,90,90]
-            - D_in: int
-                Number of input features of the surrogate model structure. It should correspond to
-                the activation electrode number.
-            - D_out: int
-                Number of output features of the surrogate model structure. It should correspond to
-                the readout electrode number.
-        * electrode_info: dict
-            It contains all the information required for the surrogate model about the electrodes.
-                * electrode_no: int
-                    Total number of electrodes in the device
-                * activation_electrodes: dict
-                    - electrode_no: int
-                        Number of activation electrodes used for gathering the data
-                    - voltage_ranges: list
-                        Voltage ranges used for gathering the data. It contains the ranges per
-                        electrode, where the shape is (electrode_no,2). Being 2 the minimum and
-                        maximum of the ranges, respectively.
-                * output_electrodes: dict
-                    - electrode_no : int
-                        Number of output electrodes used for gathering the data
-                    - clipping_value: list[float,float]
-                        Value used to apply a clipping to the sampling data within the specified
-                        values.
-                    - amplification: float
-                        Amplification correction factor used in the device to correct the
-                        amplification applied to the output current in order to convert it into
-                        voltage before its readout.
-        * training_configs: dict
-            A copy of the configurations used for training the surrogate model.
-        * sampling_configs : dict
-            A copy of the configurations used for gathering the training data.
+        1. model_structure: dict
+        The definition of the internal structure of the surrogate model, which is typically five
+        fully-connected layers of 90 nodes each.
+
+        1.1 hidden_sizes : list
+        A list containing the number of nodes of each layer of the surrogate model.
+        E.g., [90,90,90,90,90]
+
+        1.2 D_in: int
+        Number of input features of the surrogate model structure. It should correspond to
+        the activation electrode number.
+
+        1.3 D_out: int
+        Number of output features of the surrogate model structure. It should correspond to
+        the readout electrode number.
+
+        2. electrode_info: dict
+        It contains all the information required for the surrogate model about the electrodes.
+        2.1 electrode_no: int
+        Total number of electrodes in the device
+
+        2.2 activation_electrodes: dict
+        2.2.1 electrode_no: int
+        Number of activation electrodes used for gathering the data
+        
+        2.2.2 voltage_ranges: list
+        Voltage ranges used for gathering the data. It contains the ranges per
+        electrode, where the shape is (electrode_no,2). Being 2 the minimum and
+        maximum of the ranges, respectively.
+
+        2.3 output_electrodes: dict
+
+        2.3.1 electrode_no : int
+        Number of output electrodes used for gathering the data
+
+        2.3.2 clipping_value: list[float,float]
+        Value used to apply a clipping to the sampling data within the specified
+        values.
+
+        2.3.3 amplification: float
+        Amplification correction factor used in the device to correct the
+        amplification applied to the output current in order to convert it into
+        voltage before its readout.
+
+        3. training_configs: dict
+        A copy of the configurations used for training the surrogate model.
+
+        4. sampling_configs : dict
+        A copy of the configurations used for gathering the training data.
     dataloaders :  list
         A list containing a single PyTorch Dataloader containing the training dataset.
     criterion : <method>
@@ -521,12 +548,4 @@ def to_device(inputs: torch.Tensor) -> torch.Tensor:
     """
     if inputs.device != TorchUtils.get_device():
         inputs = inputs.to(device=TorchUtils.get_device())
-    return inputs
-
-
-# if __name__ == "__main__":
-#     from brainspy.utils.io import load_configs
-
-#     configs = load_configs("configs/training/smg_configs_template.yaml")
-
-#     generate_surrogate_model(configs)
+    return inputs
\ No newline at end of file
